####################################################################################################
# ------------------------------------------------------------------------------------------------ #
#                                                                                                  #
#        Sugar Factory Task (R Script)                                                             #
#        original publication: Dianne C. Berry & Donald E. Broadbent, 1984                         #
#                                                                                                  #
# ------------------------------------------------------------------------------------------------ #
####################################################################################################

# (c) 2017 Martin Zauser                                                                           #
#     Biological Information Processing Group, BIOMS / BioQuant, Heidelberg University, Germany    #
# Author: Martin Zauser                                                                            #
# Adress: Im Neuenheimer Feld 267, 69100 Heidelberg                                                #
# Email:  martin.zauser@bioquant.uni-heidelberg.de                                                 # 

####################################################################################################

# tested with: R version 3.4.1 (2017-06-30)                                                        #
#              Platform: x86_64-apple-darwin15.6.0                                                 #
#              OS:       darwin15.6.0 (MacOS High Sierra version 10.13.1)                          #
#              R Studio: 1.0.143                                                                   #

####################################################################################################
# ------------------------------------------------------------------------------------------------ #
#  Note: if you want to clean up the workspace before executing the R script you can use           #
#  the following R command for deleting the workspace (all variables/objects !!!):                 # 
#  rm(list=ls(all=TRUE))                                                                           #
# ------------------------------------------------------------------------------------------------ #
####################################################################################################



####################################################################################################
# ------------------------------------------------------------------------------------------------ #
#                                                                                                  #
# sugar factory task                                                                               #
#                                                                                                  #
# init task: sf()                                                                                  #
# then for each round: sf(n)                                                                       #
#     with n beeing the number of workers for this round (n = integer within the range of 1 to 12) #
#                                                                                                  #
# ------------------------------------------------------------------------------------------------ #
####################################################################################################
sf <- function(x=NA, reset=FALSE, initval=6, rounds=40) {
  # no argument: => reset game
  if (is.na(x)) {
    reset <- TRUE
  }
  if (reset) {
    sfval <- initval
    sfcount <- 0
    sfsuccess <- 0
    sfrounds <- rounds
    assign("sfval", sfval, envir = .GlobalEnv)
    assign("sfcount", sfcount, envir = .GlobalEnv)
    assign("sfsuccess", sfsuccess, envir = .GlobalEnv)
    assign("sfrounds", sfrounds, envir = .GlobalEnv)
    message(paste0("Start new game: initial value for sugar production: ", sfval * 1000, " metric tons, rounds: ", rounds, "."))
  } else {
    # init production
    if (exists("sfval") == FALSE) {
      sfval <- initval
      sfcount <- 0
      sfsuccess <- 0
      sfrounds <- rounds
      assign("sfrounds", sfrounds, envir = .GlobalEnv)
    }
    # check value
    x <- round(x)
    if ((x < 1) || (x > 12)) {
      message("Only worker values between 1 and 12 allowed.")
      return(0)
    }
    # calculate production
    sfcount <- sfcount + 1
    sfval <- min(max(2 * x - sfval + floor(runif(1,1,4)) - 2, 1), 12)
    success <- FALSE
    if ((sfval >= 8) && (sfval <= 10)) {
      success <- TRUE
      sfsuccess <- sfsuccess +1
    }
    # make production value global
    assign("sfval", sfval, envir = .GlobalEnv)
    assign("sfcount", sfcount, envir = .GlobalEnv)
    assign("sfsuccess", sfsuccess, envir = .GlobalEnv)
    text <- paste0("Sugar production is ", sfval * 1000, " metric tons.")
    if (success) {
      message(paste0(text, " Good job!"))
    } else {
      cat(text,"\n") 
    }           
    if (sfcount == sfrounds) {
      message(paste0("Well done. You were ", round(sfsuccess / sfcount * 100) , " % on target (", sfsuccess, " of ", sfrounds, " rounds produced between 8.000 and 10.000 tons of sugar."))
      sfval <- initval
      sfcount <- 0
      sfsuccess <- 0
      assign("sfval", sfval, envir = .GlobalEnv)
      assign("sfcount", sfcount, envir = .GlobalEnv)
      assign("sfsuccess", sfsuccess, envir = .GlobalEnv)
    }
  }
}